import {Component, HostListener, OnDestroy, OnInit} from '@angular/core';
import {LecturerInfoService} from 'src/app/services/lecturer-info.service';
import {LecturerDataService} from 'src/app/services/lecturer-data.service';
import {Router} from '@angular/router';
import {MinutePaperDao} from 'src/app/dao/minute-paper.dao';
import {GridOptions} from 'ag-grid-community';
import {ViewSubmissionsButtonComponent} from 'src/app/GridComponents/ViewSubmissionsButton/view-submissions-button.component';
import {DeletePaperButtonComponent} from 'src/app/GridComponents/DeletePaperButton/delete-paper-button.component';
import {ViewPaperButtonComponent} from 'src/app/GridComponents/ViewPaperButton/view-paper-button.component';
import {GenerateAnalysisButtonComponent} from 'src/app/GridComponents/GenerateAnalysisButton/generate-analysis-button.component';

@Component({
  selector: 'app-lecturer-paperview',
  templateUrl: './lecturer-paper-view.component.html',
  styleUrls: ['./lecturer-paper-view.component.css']
})
export class LecturerPaperViewComponent implements OnInit, OnDestroy {

  private gridApi;
  private gridColumnApi;
  public classCode: String;
  public lecturerId: String;
  public classPapers: any;
  public gridOptions: GridOptions;
  public columnDefs = [
    {headerName: 'Class Code', field: 'classCode', sortable: true, lockPosition: true, resizable: true},
    {headerName: 'Paper Name', field: 'name', sortable: true, lockPosition: true, resizable: true},
    {headerName: 'Topic', field: 'topic', sortable: true, lockPosition: true, filter: true, resizable: true},
    {headerName: 'Anonymous', field: 'anonymous', sortable: true, lockPosition: true},
    {headerName: 'Creation Date', field: 'creationDate', sortable: true, lockPosition: true},
    {headerName: 'Paper', field: '', cellRendererFramework: ViewPaperButtonComponent, lockPosition: true},
    {headerName: 'Response', field: '', cellRendererFramework: ViewSubmissionsButtonComponent, lockPosition: true},
    {headerName: 'Word-cloud', field: '', cellRendererFramework: GenerateAnalysisButtonComponent , lockPosition: true},
    {headerName: 'Delete', field: '', cellRendererFramework: DeletePaperButtonComponent, lockPosition: true},
  ];

  constructor(private lecturerInfoService: LecturerInfoService,
              private lecturerDataService: LecturerDataService,
              private router: Router) {
  }

  ngOnInit(): void {
    this.classCode = this.lecturerDataService.classCode;
    this.lecturerId = sessionStorage.getItem('username');
    this.gridOptions = {
      context: {componentParent: this},
      suppressHorizontalScroll: false,
    };
    this.classPapers = this.lecturerInfoService.getClassPapers(this.lecturerId, this.classCode);
  }

  ngOnDestroy(): void {
    this.classPapers = [];
  }

  public viewPaper(paper: MinutePaperDao) {
    this.lecturerDataService.currentPaper = paper;
    this.router.navigate(['/view-paper']);
    this.ngOnDestroy();
  }

  public goToSubmissionsView(paper: MinutePaperDao) {
    this.lecturerDataService.currentPaper = paper;
    this.router.navigate(['/submissions-view']);
    this.ngOnDestroy();
  }

  public generateAnalysis(paper: MinutePaperDao) {
    this.lecturerDataService.currentPaper = paper;
    this.router.navigate(['/analysis-view']);
    this.ngOnDestroy();
  }

  public removePaper(paper: MinutePaperDao, transaction: any) {
    const choice = confirm('Are you sure you want to delete this paper?\n' +
      'Deleting this paper will result in all responses being destroyed!');
    if (choice) {
      this.gridOptions.api.updateRowData(transaction);
      this.lecturerInfoService.deletePaper(paper.paperId).subscribe();
    }
  }

  public onGridReady(params) {
    this.gridApi = params.api;
    this.gridColumnApi = params.columnApi;
    params.api.sizeColumnsToFit();
  }

  @HostListener('window:resize')
  public onResize() {
    if (!this.gridApi) {
      return;
    }
    setTimeout(() => this.gridApi.sizeColumnsToFit());
  }

  public back() {
    this.router.navigate(['/ldashboard']);
    this.ngOnDestroy();
  }

}
